/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 */
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "avrdude.h"
#include "settings.h"
#include "dialogs.h"

AvrDudeOptions *
avrdude_options_new ()
{
        AvrDudeOptions *options = g_new(AvrDudeOptions, 1);
        
        options->part = g_string_new ("");
        options->config_file = g_string_new ("");
        options->programmer = g_string_new ("");
        options->port = g_string_new ("");
        options->memtype = g_string_new ("");
        options->filename = g_string_new ("");
        options->format = 'a';
        options->disable_auto_erase = FALSE;
        options->override_invalid_signature = FALSE;
        options->perform_chip_erase = FALSE;
        options->do_not_verify = FALSE;  
        
        return options;
}

void 
avrdude_options_free (AvrDudeOptions *options)
{
        if (options) 
        {
                g_string_free (options->part, TRUE);
                g_string_free (options->config_file, TRUE);
                g_string_free (options->programmer, TRUE);
                g_string_free (options->port, TRUE);
                g_string_free (options->memtype, TRUE);
                g_string_free (options->filename, TRUE);
        }      
}

void 
avrdude_options_set_part (AvrDudeOptions *options, const gchar* value)
{
        options->part = g_string_assign (options->part, value);
}

void 
avrdude_options_set_config_file (AvrDudeOptions *options, const gchar* value)
{
        options->config_file = g_string_assign (options->config_file, value);
}

void 
avrdude_options_set_programmer (AvrDudeOptions *options, const gchar* value)
{
        options->programmer = g_string_assign (options->programmer, value);
}

void 
avrdude_options_set_port (AvrDudeOptions *options, const gchar* value)
{
        options->port = g_string_assign (options->port, value);
}

void 
avrdude_options_set_memtype (AvrDudeOptions *options, const gchar* value)
{
        options->memtype = g_string_assign (options->memtype, value);
}

void 
avrdude_options_set_filename (AvrDudeOptions *options, const gchar* value)
{
        options->filename = g_string_assign (options->filename, value);
}

void 
avrdude_options_set_format (AvrDudeOptions *options, const gchar* value)
{
        if (g_str_equal (value, AVRDUDE_FORMAT_INTEL_HEX)) 
                options->format = 'i';
                
        else if (g_str_equal (value, AVRDUDE_FORMAT_MOTOROLA)) 
                options->format = 'm';

        else if (g_str_equal (value, AVRDUDE_FORMAT_RAW_BINARY)) 
                options->format = 'r';
                
        else options->format = 'a';
}

gboolean 
avrdude_run (GString *command, GtkTextView *output_text)
{
        if (command == NULL) return FALSE;

        FILE *fp;
        gchar *tmp_file;
        gint fp_tmp;
        gint exit_status;
        GError *err;
        
        /* create and open a tmp file to output avrdude to */
        fp_tmp = g_file_open_tmp(NULL, &tmp_file, &err);
        
        if (!fp_tmp)
        {
                message_dialog (GTK_MESSAGE_ERROR, 
                                _("Could not create temprorary file."));
                g_warning (_("Could not create temporary file: %s"),
                                err->message);
                g_error_free (err);
                return FALSE;
        }
        
        command = g_string_append (command, "2> ");
        command = g_string_append (command, tmp_file);
        
        /* execute command */
        fp = popen(command->str, "r"); 
	if (fp == NULL)
	{
		message_dialog (GTK_MESSAGE_ERROR, 
                                _("Could not open pipe."));
                g_warning (_("Could not open pipe: %s"),
                                command->str);
                        
                /* clean up */        
                g_free (tmp_file);
                close (fp_tmp);
                return FALSE;
	}
	
        exit_status = pclose (fp);
        
        load_file_into_textview (tmp_file, output_text);
        
        /* clean up */
        g_free(tmp_file);
        close(fp_tmp);
        
        if (exit_status != 0)
        {
                message_dialog (GTK_MESSAGE_ERROR, 
                                _("There was an error running AvrDude."));
                g_warning (_("There was an error running AvrDude. Exit code: %d"),
                                exit_status);
                return FALSE;
        }
        
        return TRUE;
}

GString *
avrdude_get_command (AvrDudeOptions *options, gchar op)
{
        GString *empty = g_string_new ("");
        
        if (g_string_equal(options->part, empty))
        {
                message_dialog(GTK_MESSAGE_ERROR,
                                _("You must specify a part from the 'AvrDude' -> 'Configure' menu."));
                g_string_free (empty, TRUE);
                return NULL;
        }
        
        
        GString *command = g_string_new ("");
        g_string_printf (command, "%s %s %s %s %s:%c:'%s':%c %s %s ",
                        AVRDUDE_COMMAND,
                        AVRDUDE_OPTION_PART,
                        options->part->str,
                        AVRDUDE_OPTION_MEMORY_OP_SPEC,
                        options->memtype->str,
                        op,
                        options->filename->str,
                        options->format,
                        AVRDUDE_OPTION_DISABLE_SAFEMODE,
                        AVRDUDE_OPTION_QUELL_OUTPUT);

        
        if (!g_string_equal(options->port, empty))
        {
                command = g_string_append (command, AVRDUDE_OPTION_PORT" ");
                command = g_string_append (command, options->port->str);
                command = g_string_append (command, " ");
        }
        
        if (!g_string_equal(options->programmer, empty))
        {
                command = g_string_append (command, AVRDUDE_OPTION_PROGRAMMER" ");
                command = g_string_append (command, options->programmer->str);
                command = g_string_append (command, " ");
        }
        
        if (!g_string_equal(options->config_file, empty))
        {
                command = g_string_append (command, AVRDUDE_OPTION_CONFIG_FILE" ");
                command = g_string_append (command, options->config_file->str);
                command = g_string_append (command, " ");
        }
        
        if (options->disable_auto_erase)
        {
                command = g_string_append (command, 
                                AVRDUDE_OPTION_DISABLE_AUTO_ERASE" ");     
        }
        
        if (options->override_invalid_signature)
        {
                command = g_string_append (command, 
                                AVRDUDE_OPTION_OVERRIDE_INVALID_SIGNATURE" ");     
        }
        
        if (options->perform_chip_erase)
        {
                command = g_string_append (command, 
                                AVRDUDE_OPTION_PERFORM_CHIP_ERASE" ");     
        }
        
        if (options->do_not_verify)
        {
                command = g_string_append (command, 
                                AVRDUDE_OPTION_DO_NOT_VERIFY" ");     
        }

        g_string_free (empty, TRUE);
        
        return command;
}

void 
avrdude_debug_dump (AvrDudeOptions *options)
{
        g_debug ("avrdude_debug_dump() \noptions->part=%s\noptions->config_file=%s\noptions->programmer=%s\noptions->port=%s\noptions->memtype=%s\noptions->filename=%s\noptions->format=%c\noptions->disable_auto_erase=%d\noptions->override_invalid_signature=%d\noptions->perform_chip_erase=%d\noptions->do_not_verify=%d",
                        options->part->str,
                        options->config_file->str,
                        options->programmer->str,
                        options->port->str,
                        options->memtype->str,
                        options->filename->str,
                        options->format,
                        options->disable_auto_erase,
                        options->override_invalid_signature,
                        options->perform_chip_erase,
                        options->do_not_verify); 
}


