/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <gnome.h>
#include <glade/glade.h>

#include "callbacks.h"
#include "settings.h"
#include "dialogs.h"
#include "avrdude.h"

void 
on_app_window_destroy (GtkObject *object, gpointer user_data)
{
        gtk_main_quit ();
}

gint 
on_app_window_delete_event (GtkWidget *widget, gpointer user_data)
{
        save_window_position(GTK_WINDOW(widget));
        return 0;
}

void
on_open_activate (GtkWidget *widget, gpointer user_data)
{
        CallBackData *cb_data = (CallBackData *)user_data;
        
        const gchar *filename = open_file_dialog();
        
        if (!filename) return;
               
        /* set filename in options structure */
        avrdude_options_set_filename (cb_data->options, filename);
        set_appbar_default (cb_data->gxml, cb_data->options);         
}

void 
on_preferences_activate (GtkWidget *widget, gpointer user_data)
{
        CallBackData *cb_data = (CallBackData *)user_data;

        preferences_dialog ();
        
        apply_preferences (cb_data->gxml);
}

void 
on_quit_activate (GtkWidget *widget, gpointer user_data)
{
        save_window_position(GTK_WINDOW(user_data));
        gtk_main_quit();
}

void
on_read_activate (GtkWidget *widget, gpointer user_data)
{
        CallBackData *cb_data = (CallBackData *)user_data;
        GtkWidget *output_text;
        GtkWidget *running_dialog;
        const gchar *filename = save_file_dialog();
        
        if (!filename) return;
        avrdude_options_set_filename (cb_data->options, filename);
        
        /* get the avrdude command string */
        GString *command = avrdude_get_command(cb_data->options, AVRDUDE_OP_READ);
        g_assert(command);
        
        /* show "running..." dialog */
        running_dialog = show_running_dialog(AVRDUDE_OP_READ);
        
        /* execute avrdude command */
        output_text = glade_xml_get_widget (cb_data->gxml, "output_text");
        if (!avrdude_run (command, GTK_TEXT_VIEW(output_text))) 
        {
                avrdude_options_set_filename (cb_data->options, "");
                g_string_free (command, TRUE);
                gtk_widget_destroy (running_dialog);
                return;
        }
        g_string_free (command, TRUE);
        
     
        /* set filename in options structure */
        avrdude_options_set_filename (cb_data->options, filename);
        set_appbar_default (cb_data->gxml, cb_data->options);
        
        /* kill "running..." dialog */
        gtk_widget_destroy (running_dialog);
}

void
on_write_activate (GtkWidget *widget, gpointer user_data)
{
        CallBackData *cb_data = (CallBackData *)user_data;
        GtkWidget *output_text;
        GtkWidget *running_dialog;
        
        if (cb_data->options->filename->len == 0) 
        {
                message_dialog(GTK_MESSAGE_ERROR,
                                "You must first open a file to write to the device");
                return;
        }        
        
        /* get the avrdude command string */
        GString *command = avrdude_get_command(cb_data->options, AVRDUDE_OP_WRITE);
        g_assert(command);
        
        /* show "running..." dialog */
        running_dialog = show_running_dialog(AVRDUDE_OP_WRITE);
        
        /* execute avrdude command */
        output_text = glade_xml_get_widget (cb_data->gxml, "output_text");
        if (!avrdude_run (command, GTK_TEXT_VIEW(output_text))) 
        {
                avrdude_options_set_filename (cb_data->options, "");
                g_string_free (command, TRUE);
                gtk_widget_destroy (running_dialog);
                return;
        }
        g_string_free (command, TRUE);
                
        /* kill "running..." dialog */
        gtk_widget_destroy (running_dialog);
}

void
on_verify_activate (GtkWidget *widget, gpointer user_data)
{
        CallBackData *cb_data = (CallBackData *)user_data;
        GtkWidget *output_text;
        GtkWidget *running_dialog;
        
        if (cb_data->options->filename->len == 0) 
        {
                message_dialog(GTK_MESSAGE_ERROR,
                                "You must first open a file to verify.");
                return;
        }        
        
        /* get the avrdude command string */
        GString *command = avrdude_get_command(cb_data->options, AVRDUDE_OP_VERIFY);
        g_assert(command);
        
        /* show "running..." dialog */
        running_dialog = show_running_dialog(AVRDUDE_OP_VERIFY);
        
        /* execute avrdude command */
        output_text = glade_xml_get_widget (cb_data->gxml, "output_text");
        if (!avrdude_run (command, GTK_TEXT_VIEW(output_text))) 
        {
                avrdude_options_set_filename (cb_data->options, "");
                g_string_free (command, TRUE);
                gtk_widget_destroy (running_dialog);
                return;
        }
        g_string_free (command, TRUE);
                
        /* kill "running..." dialog */
        gtk_widget_destroy (running_dialog);
}

void
on_configure_activate (GtkWidget *widget, gpointer user_data)
{
        CallBackData *cb_data = (CallBackData *)user_data;
        
        configure_dialog (cb_data->options);
        
        set_appbar_default (cb_data->gxml, cb_data->options);
}

void
on_about_activate (GtkWidget *widget, gpointer user_data)
{
        about_dialog ();
}



